//
//  sbcexample.c
//
// This simple example creates an InterfaceKit handle, hooks the event
// handlers and opens it.
// It reads the status of the inputs and outputs then waits for an InterfaceKit
// to be attached and waits for events to be fired.
// Digital output 0 will trigger on when any sensor value is greater than 500.
//
// Modified by Kat Dornian on 2014-03-26.
//
// Copyright 2014 Phidgets Inc.
// This work is licensed under the Creative Commons Attribution 2.5 Canada License.
// To view a copy of this license, visit http://creativecommons.org/licenses/by/2.5/ca/
//

#include <stdio.h>
#include <unistd.h>
#include <phidget21.h>

int CCONV AttachHandler(CPhidgetHandle IFK, void *userptr)
{
	int serialNo;
	const char *name;
    
	CPhidget_getDeviceName(IFK, &name);
	CPhidget_getSerialNumber(IFK, &serialNo);
    
	return 0;
}

int CCONV DetachHandler(CPhidgetHandle IFK, void *userptr)
{
	int serialNo, numOutputs, i;
	const char *name;
    
	CPhidget_getDeviceName (IFK, &name);
	CPhidget_getSerialNumber(IFK, &serialNo);
    
	fprintf(stdout, "%s %10d detached!\n", name, serialNo);
    fflush(stdout);
    
	return 0;
}

int CCONV ErrorHandler(CPhidgetHandle IFK, void *userptr, int ErrorCode, const char *unknown)
{
	fprintf(stderr, "Error handled. %d - %s", ErrorCode, unknown);
    fflush(stderr);
    
	return 0;
}

//callback that will run if an input changes.
//Index - Index of the input that generated the event, State - boolean (0 or 1) representing the input state (on or off)
int CCONV InputChangeHandler(CPhidgetInterfaceKitHandle IFK, void *usrptr, int Index, int State)
{
	fprintf(stdout, "Digital Input: %d > State: %d\n", Index, State);
    fflush(stdout);
    
	return 0;
}

//callback that will run if an output changes.
//Index - Index of the output that generated the event, State - boolean (0 or 1) representing the output state (on or off)
int CCONV OutputChangeHandler(CPhidgetInterfaceKitHandle IFK, void *usrptr, int Index, int State)
{
	fprintf(stdout, "Digital Output: %d > State: %d\n", Index, State);
    fflush(stdout);
    
	return 0;
}

//callback that will run if the sensor value changes by more than the OnSensorChange trigger.
//Index - Index of the sensor that generated the event, Value - the sensor read value
int CCONV SensorChangeHandler(CPhidgetInterfaceKitHandle IFK, void *usrptr, int Index, int Value)
{
    //When any sensor changes, if the value is more than 500 the LED will turn on
	fprintf(stdout, "Sensor: %d > Value: %d\n", Index, Value);
    
    if (Value > 500) {
        CPhidgetInterfaceKit_setOutputState(IFK, 0, PTRUE);
    }
    else {
        CPhidgetInterfaceKit_setOutputState(IFK, 0, PFALSE);
    }
    
    fflush(stdout);
    
	return 0;
}

//Display the properties of the attached phidget to the screen.  We will be displaying the name, serial number and version of the attached device.
//Will also display the number of inputs, outputs, and analog inputs on the interface kit as well as the state of the ratiometric flag
//and the current analog sensor sensitivity.
int display_properties(CPhidgetInterfaceKitHandle phid)
{
	int serialNo, version, numInputs, numOutputs, numSensors, triggerVal, ratiometric, i, State, Value;
	const char* ptr;
    
	CPhidget_getDeviceType((CPhidgetHandle)phid, &ptr);
	CPhidget_getSerialNumber((CPhidgetHandle)phid, &serialNo);
	CPhidget_getDeviceVersion((CPhidgetHandle)phid, &version);
    
	CPhidgetInterfaceKit_getInputCount(phid, &numInputs);
	CPhidgetInterfaceKit_getOutputCount(phid, &numOutputs);
	CPhidgetInterfaceKit_getSensorCount(phid, &numSensors);
	CPhidgetInterfaceKit_getRatiometric(phid, &ratiometric);
    
	fprintf(stdout, "%s\n", ptr);
	fprintf(stdout, "Serial Number: %10d\nVersion: %8d\n", serialNo, version);
	fprintf(stdout, "# Digital Inputs: %d\n# Digital Outputs: %d\n", numInputs, numOutputs);
	fprintf(stdout, "# Sensors: %d\n", numSensors);
	fprintf(stdout, "Ratiometric: %d\n", ratiometric);
    
	for(i = 0; i < numSensors; i++) {
		CPhidgetInterfaceKit_getSensorChangeTrigger (phid, i, &triggerVal);
        
		fprintf(stdout, "Sensor#: %d > Sensitivity Trigger: %d\n", i, triggerVal);
	}
	for(i = 0; i < numInputs; i++) {
		CPhidgetInterfaceKit_getInputState (phid, i, &State);
        
		fprintf(stdout, "Digital Input: %d > State: %d\n", i, State);
	}
	for(i = 0; i < numOutputs; i++) {
		CPhidgetInterfaceKit_getOutputState (phid, i, &State);
        
		fprintf(stdout, "Digital Output: %d > State: %d\n", i, State);
	}
	for(i = 0; i < numSensors; i++) {
		CPhidgetInterfaceKit_getSensorValue (phid, i, &Value);
        
		fprintf(stdout, "Sensor: %d > Value: %d\n", i, Value);
	}
    
    fflush(stdout);
	return 0;
}


int interfacekit_simple()
{
	int result, numSensors, numOutputs, i;
	const char *err;
    
	//Declare an InterfaceKit handle
	CPhidgetInterfaceKitHandle ifKit = 0;
    
	//create the InterfaceKit object
	CPhidgetInterfaceKit_create(&ifKit);
    
	//Set the handlers to be run when the device is plugged in or opened from software, unplugged or closed from software, or generates an error.
	CPhidget_set_OnAttach_Handler((CPhidgetHandle)ifKit, AttachHandler, NULL);
	CPhidget_set_OnDetach_Handler((CPhidgetHandle)ifKit, DetachHandler, NULL);
	CPhidget_set_OnError_Handler((CPhidgetHandle)ifKit, ErrorHandler, NULL);
    
	//open the interfacekit for device connections
	CPhidget_open((CPhidgetHandle)ifKit, -1);
    
	//get the program to wait for an interface kit device to be attached
	printf("Waiting for interface kit to be attached....");
    
	if((result = CPhidget_waitForAttachment((CPhidgetHandle)ifKit, 10000))) {
		CPhidget_getErrorDescription(result, &err);
		printf("Problem waiting for attachment: %s\n", err);
        
		return 0;
	}
    
	//Display the properties of the attached interface kit device
	display_properties(ifKit);
    
	//Registers a callback that will run if an input changes.
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetInterfaceKit_set_OnInputChange_Handler (ifKit, InputChangeHandler, NULL);
    
	//Registers a callback that will run if the sensor value changes by more than the OnSensorChange trig-ger.
	//Requires the handle for the IntefaceKit, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetInterfaceKit_set_OnSensorChange_Handler (ifKit, SensorChangeHandler, NULL);
    
	//Registers a callback that will run if an output changes.
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetInterfaceKit_set_OnOutputChange_Handler (ifKit, OutputChangeHandler, NULL);
    
	//keep displaying interface kit until program is ended
    while(1) {
        usleep(1000);
    }
    
	CPhidget_close((CPhidgetHandle)ifKit);
	CPhidget_delete((CPhidgetHandle)ifKit);
    
	//all done, exit
	return 0;
}

int main(int argc, char* argv[])
{
	interfacekit_simple();
	return 0;
}